@@ -3,7 +3,7 @@ import sys
 from pwncli.cli import pass_environ
 
 
-@click.command(name='example', short_help="Test command.")
+@click.command(name='example', short_help="Example command.")
 @click.option('-v', '--verbose', count=True, show_default=True, help="Show more info or not.")
 @pass_environ
 def cli(ctx, verbose):
