@@ -9,4 +9,4 @@ from pwncli.utils.shellcode import *
 from pwncli.utils.libcbox import *
 from pwncli.utils.gdb_helper import *
 from pwncli.utils.bruteforce import *
-from pwncli.utils.ropperbox import *
+from pwncli.utils.gadgetbox import *
