@@ -38,13 +38,13 @@ import subprocess
 import struct
 from pwn import unpack, pack, flat, ELF, context, which
 import click
+import time
 
 __all__ = [
     "int16",
     "int8",
     "int2",
     "int16_ex",
-    "int16_ex",
     "int8_ex",
     "int2_ex",
     "int_ex",
@@ -61,8 +61,11 @@ __all__ = [
     "errlog_exit",
     "errlog_ex_highlight_exit",
     "log_address",
+    "leak",
     "log_address_ex",
+    "leak_ex",
     "log_address_ex2",
+    "leak_ex2",
     "log_libc_base_addr",
     "log_heap_base_addr",
     "log_code_base_addr",
@@ -70,14 +73,18 @@ __all__ = [
     "one_gadget",
     "one_gadget_binary",
     "u16_ex",
+    "u24_ex",
     "u32_ex",
     "u64_ex",
     "p8_ex",
     "p16_ex",
+    "p24_ex",
     "p32_ex",
     "p64_ex",
     "p32_float",
     "p64_float",
+    "pad_ljust",
+    "pad_rjust",
     "float_hexstr2int",
     "generate_payload_for_connect",
     "recv_libc_addr",
@@ -92,7 +99,10 @@ __all__ = [
     "calc_countaddr_tcache",
     "calc_entryaddr_tcache",
     "calc_countaddr_by_entryaddr_tcache",
-    "calc_entryaddr_by_countaddr_tcache"
+    "calc_entryaddr_by_countaddr_tcache",
+    "protect_ptr",
+    "reveal_ptr",
+    "step_split"
 ]
 
 int16 = functools.partial(int, base=16)
@@ -104,10 +114,43 @@ int8_ex = lambda x: int8(x.decode()) if isinstance(x, bytes) else int8(x)
 int2_ex = lambda x: int2(x.decode()) if isinstance(x, bytes) else int2(x)
 int_ex = lambda x: int(x.decode()) if isinstance(x, bytes) else int(x)
 
-
 flat_z = functools.partial(flat, filler=b"\x00")
 
 
+def step_split(s: str or bytes, step_len: int):
+    """
+    step_split("12345678", 4) -> "1234", "5678"
+    step_split("1234567", 4) -> "1234", "567"
+    """
+    assert step_len > 0, "wrong step length!"
+    start = 0
+    end = start + step_len
+    var = None
+    while True:
+        var = s[start:end]
+        if len(var) != step_len:
+            if len(var) != 0:
+                yield var
+            break
+        yield var
+        start = end
+        end = start + step_len
+
+        
+def protect_ptr(address, next) -> int:
+    return (address >> 12) ^ next
+
+def reveal_ptr(addr) -> int:
+    """
+    addr = addr1 ^ addr2
+    addr2 = addr1 + XXX 
+    calc the heap address
+    """
+    _res = addr
+    for i in range(3):
+        _res = (_res >> 12) ^ addr
+    return _res
+
 
 def get_callframe_info(depth:int=2):
     """Get stackframe info
@@ -208,6 +251,7 @@ def log_address(desc:str, address:int):
     """
     log_ex("{} ===> {}".format(desc, hex(address)))
 
+leak = log_address
 
 def log_address_ex(variable_name:str, depth=2):
     """Log address from the variable's name by use of stack frame.
@@ -229,6 +273,7 @@ def log_address_ex(variable_name:str, depth=2):
         assert isinstance(var, int), "The address is not int!"
         log_address(variable_name, var)
 
+leak_ex = log_address_ex
 
 def log_address_ex2(variable: int, depth: int=2):
     """Log address by variable
@@ -250,6 +295,7 @@ def log_address_ex2(variable: int, depth: int=2):
             return
     errlog_exit("Cannot find variable, check your depth!")
 
+leak_ex2 = log_address_ex2
 
 def log_libc_base_addr(address:int):
     log_address("libc_base_addr", address)
@@ -278,7 +324,7 @@ def ldd_get_libc_path(filepath:str) -> str:
     try:
         out = subprocess.check_output(["ldd", filepath], encoding='utf-8').split()
         for o in out:
-            if "/libc.so" in o or "/libc-2." in o:
+            if "/libc" in o:
                 rp = os.path.realpath(o)
                 break
     except:
@@ -316,8 +362,8 @@ def one_gadget(condition:str, more=False, buildid=False):
         errlog_exit("Cannot exec one_gadget, maybe you don't install one_gadget or filename is wrong or buildid is wrong!")
 
 
-def one_gadget_binary(binary_path:str, more=False) -> int:
-    """Get all one_gadget about a elf binary file.
+def one_gadget_binary(binary_path:str, more=False):
+    """Get all one_gadget about an elf binary file.
 
     """
     binary_path = os.path.realpath(binary_path)
@@ -328,8 +374,8 @@ def one_gadget_binary(binary_path:str, more=False) -> int:
         errlog_exit("Exec ldd {} fail!".format(binary_path))
 
 
-#--------------------------------usefule function------------------------------
-def u16_ex(data: str or bytes):
+#--------------------------------useful function------------------------------
+def u16_ex(data: str or bytes) -> int:
     assert isinstance(data, (str, bytes)), "wrong data type!"
     length = len(data)
     assert length <= 2, "len(data) > 2!"
@@ -339,7 +385,17 @@ def u16_ex(data: str or bytes):
     return unpack(data, 16)
 
 
-def u32_ex(data: str or bytes):
+def u24_ex(data: str or bytes) -> int:
+    assert isinstance(data, (str, bytes)), "wrong data type!"
+    length = len(data)
+    assert length <= 3, "len(data) > 3!"
+    if isinstance(data, str):
+        data = data.encode('utf-8')
+    data = data.ljust(3, b"\x00")
+    return unpack(data, 24)
+
+
+def u32_ex(data: str or bytes) -> int:
     assert isinstance(data, (str, bytes)), "wrong data type!"
     length = len(data)
     assert length <= 4, "len(data) > 4!"
@@ -349,7 +405,7 @@ def u32_ex(data: str or bytes):
     return unpack(data, 32)
     
 
-def u64_ex(data: str or bytes):
+def u64_ex(data: str or bytes) -> int:
     length = len(data)
     assert length <= 8, "len(data) > 8!"
     assert isinstance(data, (str, bytes)), "wrong data type!"
@@ -359,22 +415,36 @@ def u64_ex(data: str or bytes):
     return unpack(data, 64)
 
 
-def p8_ex(num:int):
+def p8_ex(num:int) -> bytes:
+    if num < 0:
+        num += 1 << 8
     num &= 0xff
     return pack(num, word_size=8)
 
 
-def p16_ex(num:int):
+def p16_ex(num:int) -> bytes:
+    if num < 0:
+        num += 1 << 16
     num &= 0xffff
     return pack(num, word_size=16)
 
 
-def p32_ex(num:int):
+def p24_ex(num: int) -> bytes:
+    if num < 0:
+        num += 1 << 24
+    num &= 0xffffff
+    return pack(num, word_size=24)
+
+def p32_ex(num:int) -> bytes:
+    if num < 0:
+        num += 1 << 32
     num &= 0xffffffff
     return pack(num, word_size=32)
 
 
-def p64_ex(num:int):
+def p64_ex(num:int) -> bytes:
+    if num < 0:
+        num += 1 << 64
     num &= 0xffffffffffffffff
     return pack(num, word_size=64)
 
@@ -388,7 +458,7 @@ def p32_float(num:float, endian="little") -> bytes:
         raise RuntimeError("Wrong endian!")
         
 
-def p64_float(num:float, endian="little"):
+def p64_float(num:float, endian="little") -> bytes:
     if endian.lower() == "little":
         return struct.pack("<d", num)
     elif endian.lower() == "big":
@@ -396,8 +466,21 @@ def p64_float(num:float, endian="little"):
     else:
         raise RuntimeError("Wrong endian!")
 
+def pad_ljust(payload, psz, filler="\x00") -> bytes:
+    len_ = len(payload)
+    comple = len_ % psz
+    if comple > 0:
+        return flat(payload, filler * (psz - comple))
+    return payload
 
-def float_hexstr2int(data: str or bytes, hexstr=True, endian="little", bits=64):
+def pad_rjust(payload, psz, filler="\x00") -> bytes:
+    len_ = len(payload)
+    comple = len_ % psz
+    if comple > 0:
+        return flat(filler * (psz - comple), payload)
+    return payload
+
+def float_hexstr2int(data: str or bytes, hexstr=True, endian="little", bits=64) -> int:
     """float_hex2int('0x0.07f6d266e9fbp-1022') ---> 140106772946864"""
     endian = endian.lower()
     assert endian in ("little", "big"), "only little or big for endian!"
@@ -434,7 +517,7 @@ def float_hexstr2int(data: str or bytes, hexstr=True, endian="little", bits=64):
         errlog_exit("float_hex2int failed, check cmd: \n{}".format(cmd))
         
 
-def generate_payload_for_connect(ip: str, port: int):
+def generate_payload_for_connect(ip: str, port: int) -> bytes:
     """connect(socket_fd, buf, 0x10), generate payload of buf
     
     assert len(buf) == 0x10
@@ -447,7 +530,7 @@ def generate_payload_for_connect(ip: str, port: int):
     return pack(2, word_size=16, endianness="little") + pack(port, word_size=16, endianness="big") + pack(int_ip, word_size=32, endianness="big") + pack(0, 64)
 
 
-def recv_libc_addr(io, *, bits=64, offset=0) -> int:
+def recv_libc_addr(io, *, bits=64, offset=0, timeout=5) -> int:
     """Calcuate libc-base addr while recv '\x7f' in amd64 or '\xf7' in i386.
 
     Args:
@@ -463,7 +546,7 @@ def recv_libc_addr(io, *, bits=64, offset=0) -> int:
     """
     assert bits == 32 or bits == 64
     contains = b"\x7f" if bits == 64 else b"\xf7"
-    m = io.recvuntil(contains)
+    m = io.recvuntil(contains, timeout=timeout)
     if contains not in m:
         raise RuntimeError("Cannot get libc addr")
     if bits == 32:
@@ -471,7 +554,7 @@ def recv_libc_addr(io, *, bits=64, offset=0) -> int:
     else:
         return u64_ex(m[-6:]) - offset
 
-
+#@unused("Remove since 1.4")
 def get_flag_when_get_shell(io, use_cat:bool=True, start_str:str="flag{", timeout=10):
     """Get flag while get a shell
 
@@ -489,6 +572,7 @@ def get_flag_when_get_shell(io, use_cat:bool=True, start_str:str="flag{", timeou
     else:
         errlog_ex_highlight("Cannot get flag")
 
+#@unused("Remove since 1.4")
 def get_flag_by_recv(io, flag_reg: str="flag{", timeout=10):
     get_flag_when_get_shell(io,use_cat=False, start_str=flag_reg, timeout=timeout)
 
@@ -516,7 +600,7 @@ def get_segment_base_addr_by_proc_maps(pid:int, filename:str=None) -> dict:
     ld_flag = 0
 
     for r in res:
-        rc = re.compile(r"^([0123456789abcdef]{6,14})-([0123456789abcdef]{6,14})", re.S)
+        rc = re.compile(r"^([0-9a-f]{6,14})-([0-9a-f]{6,14})", re.S)
         rc = rc.findall(r)
         if len(rc) != 1 or len(rc[0]) != 2:
             continue
@@ -525,10 +609,10 @@ def get_segment_base_addr_by_proc_maps(pid:int, filename:str=None) -> dict:
         if (filename is not None) and (not code_flag) and filename in r:
             code_flag = 1
             _d['code'] = start_addr
-        elif (not libc_flag) and ("/libc-2." in r or "/libc.so" in r):
+        elif (not libc_flag) and ("/libc" in r):
             libc_flag = 1
             _d['libc'] = start_addr
-        elif (not ld_flag) and ("/ld-2." in r):
+        elif (not ld_flag) and ("/ld" in r):
             ld_flag = 1
             _d['ld'] = start_addr
         elif "heap" in r:
